"""
PHCpack offers numerical Schubert calculus, exported here.
"""

def prompt_for_dimensions():
    """
    Returns the triplet (m,p,q),
    where m is the dimension of the input planes,
    p is the dimension of the output planes, and
    q is the degree of the maps.
    """
    mdim = raw_input('give the dimension of the input planes : ')
    pdim = raw_input('give the dimension of the output planes : ')
    qdeg = raw_input('give the degree of the solution maps : ')
    return (int(mdim), int(pdim), int(qdeg))

def pieri_root_count(mdim, pdim, qdeg, verbose=True):
    r"""
    Computes the number of *pdim*-plane producing maps of
    degree *qdeg* that meet *mdim*-planes
    at mdim*pdim + qdeg*(mdim+pdim) points.
    """
    from phcpy.phcpy2c2 import py2c_schubert_pieri_count
    from phcpy.phcpy2c2 import py2c_schubert_localization_poset
    root_count = py2c_schubert_pieri_count(mdim, pdim, qdeg)
    if verbose:
        print 'Pieri root count for', (mdim, pdim, qdeg), 'is', root_count
        poset = py2c_schubert_localization_poset(mdim, pdim, qdeg)
        print 'the localization poset :'
        print poset
    return root_count

def resolve_schubert_conditions(ndim, kdim, brackets, verbose=True):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The *brackets* is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    On return is the formal root count, which is sharp for general flags.
    and the coordinates of the flags, stored row wise in a list
    of real and imaginary parts.
    """
    from phcpy.phcpy2c2 import py2c_schubert_resolve_conditions as resolve
    nbc = len(brackets)
    cds = ''
    for bracket in brackets:
        for num in bracket:
            cds = cds + ' ' + str(num)
    # print 'the condition string :', cds
    roco = resolve(ndim, kdim, nbc, len(cds), cds, int(verbose))
    return roco

def standard_littlewood_richardson_homotopies(ndim, kdim, brackets, \
    verbose=True, vrfcnd=False, minrep=True, tosqr=False, outputfilename=''):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The parameters *ndim* and *kdim* give values for n and k respectively.
    The parameter *brackets* is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    The Littlewood-Richardson homotopies compute k-planes that
    meet the flags at spaces of dimensions prescribed by the brackets,
    in standard double precision.  Four options are passed as Booleans:

    *verbose*: for adding extra output during computations,

    *vrfcnd*: for extra diagnostic verification of Schubert conditions,

    *minrep*: for a minimial representation of the problem formulation,

    *tosqr*: to square the overdetermined systems.

    On return is a 4-tuple.  The first item of the tuple is the
    formal root count, sharp for general flags, then as second
    item the coordinates of the flags.  The coordinates of the
    flags are stored row wise in a list of real and imaginary parts.
    The third and fourth item of the tuple on return are respectively
    the polynomial system that has been solved and its solutions.
    The length of the list of solution should match the root count.
    """
    from phcpy.phcpy2c2 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c2 \
        import py2c_schubert_standard_littlewood_richardson_homotopies \
        as stlrhom
    from phcpy.interface import load_standard_solutions, load_standard_system
    py2c_solcon_clear_standard_solutions()
    nbc = len(brackets)
    cds = ''
    for bracket in brackets:
        for num in bracket:
            cds = cds + ' ' + str(num)
    # print 'the condition string :', cds
    (roco, sflags) = stlrhom(ndim, kdim, nbc, len(cds), cds, \
        int(verbose), int(vrfcnd), int(minrep), int(tosqr), \
        len(outputfilename), outputfilename)
    rflags = eval(sflags)
    flgs = []
    for k in range(len(rflags)/2):
        flgs.append(complex(rflags[2*k], rflags[2*k+1]))
    fsys = load_standard_system()
    sols = load_standard_solutions()
    return (roco, flgs, fsys, sols)

def dobldobl_littlewood_richardson_homotopies(ndim, kdim, brackets, \
    verbose=True, vrfcnd=False, minrep=True, tosqr=False, outputfilename=''):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The parameters *ndim* and *kdim* give values for n and k respectively.
    The parameter *brackets* is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    The Littlewood-Richardson homotopies compute k-planes that
    meet the flags at spaces of dimensions prescribed by the brackets,
    in double double precision.  Four options are passed as Booleans:

    *verbose*: for adding extra output during computations,

    *vrfcnd*: for extra diagnostic verification of Schubert conditions,

    *minrep*: for a minimial representation of the problem formulation,

    *tosqr*: to square the overdetermined systems.

    On return is a 4-tuple.  The first item of the tuple is the
    formal root count, sharp for general flags, then as second
    item the coordinates of the flags.  The coordinates of the
    flags are stored row wise in a list of real and imaginary parts.
    The third and fourth item of the tuple on return are respectively
    the polynomial system that has been solved and its solutions.
    The length of the list of solution should match the root count.
    """
    from phcpy.phcpy2c2 import py2c_solcon_clear_dobldobl_solutions
    from phcpy.phcpy2c2 \
        import py2c_schubert_dobldobl_littlewood_richardson_homotopies \
        as ddlrhom
    from phcpy.interface import load_dobldobl_solutions, load_dobldobl_system
    py2c_solcon_clear_dobldobl_solutions()
    nbc = len(brackets)
    cds = ''
    for bracket in brackets:
        for num in bracket:
            cds = cds + ' ' + str(num)
    # print 'the condition string :', cds
    (roco, sflags) = ddlrhom(ndim, kdim, nbc, len(cds), cds, \
        int(verbose), int(vrfcnd), int(minrep), int(tosqr), \
        len(outputfilename), outputfilename)
    rflags = eval(sflags)
    flgs = []
    for k in range(len(rflags)/4):
        flgs.append(complex(rflags[2*k], rflags[2*k+2]))
    fsys = load_dobldobl_system()
    sols = load_dobldobl_solutions()
    return (roco, flgs, fsys, sols)

def quaddobl_littlewood_richardson_homotopies(ndim, kdim, brackets, \
    verbose=True, vrfcnd=False, minrep=True, tosqr=False, outputfilename=''):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The parameters *ndim* and *kdim* give values for n and k respectively.
    The parameter brackets is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    The Littlewood-Richardson homotopies compute k-planes that
    meet the flags at spaces of dimensions prescribed by the brackets,
    in quad double precision.  Four options are passed as Booleans:

    *verbose*: for adding extra output during computations,

    *vrfcnd*: for extra diagnostic verification of Schubert conditions,

    *minrep*: for a minimial representation of the problem formulation,

    *tosqr*: to square the overdetermined systems.

    On return is a 4-tuple.  The first item of the tuple is the
    formal root count, sharp for general flags, then as second
    item the coordinates of the flags.  The coordinates of the
    flags are stored row wise in a list of real and imaginary parts.
    The third and fourth item of the tuple on return are respectively
    the polynomial system that has been solved and its solutions.
    The length of the list of solution should match the root count.
    """
    from phcpy.phcpy2c2 import py2c_solcon_clear_quaddobl_solutions
    from phcpy.phcpy2c2 \
        import py2c_schubert_quaddobl_littlewood_richardson_homotopies \
        as qdlrhom
    from phcpy.interface import load_quaddobl_solutions, load_quaddobl_system
    py2c_solcon_clear_quaddobl_solutions()
    nbc = len(brackets)
    cds = ''
    for bracket in brackets:
        for num in bracket:
            cds = cds + ' ' + str(num)
    # print 'the condition string :', cds
    (roco, sflags) = qdlrhom(ndim, kdim, nbc, len(cds), cds, \
        int(verbose), int(vrfcnd), int(minrep), int(tosqr), \
        len(outputfilename), outputfilename)
    rflags = eval(sflags)
    flgs = []
    for k in range(len(rflags)/8):
        flgs.append(complex(rflags[2*k], rflags[2*k+4]))
    fsys = load_quaddobl_system()
    sols = load_quaddobl_solutions()
    return (roco, flgs, fsys, sols)

def littlewood_richardson_homotopies(ndim, kdim, brackets, \
    verbose=True, vrfcnd=False, minrep=True, tosqr=False, \
    precision='d', outputfilename=''):
    r"""
    In n-dimensional space we consider k-dimensional planes,
    subject to intersection conditions represented by brackets.
    The parameters *ndim* and *kdim* give values for n and k respectively.
    The parameter *brackets* is a list of brackets.  A bracket is a list
    of as many natural numbers (in the range 1..*ndim*) as *kdim*.
    The Littlewood-Richardson homotopies compute k-planes that
    meet the flags at spaces of dimensions prescribed by the brackets.
    Four options are passed as Booleans:

    *verbose*: for adding extra output during computations,

    *vrfcnd*: for extra diagnostic verification of Schubert conditions,

    *minrep*: for a minimial representation of the problem formulation,

    *tosqr*: to square the overdetermined systems.

    On return is a 4-tuple.  The first item of the tuple is the
    formal root count, sharp for general flags, then as second
    item the coordinates of the flags.  The coordinates of the
    flags are stored row wise in a list of real and imaginary parts.
    The third and fourth item of the tuple on return are respectively
    the polynomial system that has been solved and its solutions.
    The length of the list of solution should match the root count.
    """
    if(precision == 'd'):
        return standard_littlewood_richardson_homotopies(ndim, kdim, \
                  brackets, verbose, vrfcnd, minrep, tosqr, outputfilename)
    elif(precision == 'dd'):
        return dobldobl_littlewood_richardson_homotopies(ndim, kdim, \
                  brackets, verbose, vrfcnd, minrep, tosqr, outputfilename)
    elif(precision == 'qd'):
        return quaddobl_littlewood_richardson_homotopies(ndim, kdim, \
                  brackets, verbose, vrfcnd, minrep, tosqr, outputfilename)
    else:
        print 'wrong level of precision, use d, dd, or qd'

def random_complex_matrix(nbrows, nbcols):
    r"""
    Returns a random *nbrows*-by-*nbcols* matrix
    with randomly generated complex coefficients
    on the unit circle, as a list of *rows*.
    """
    from math import pi, sin, cos
    from random import uniform as u
    result = []
    for i in range(0, nbrows):
        angles = [u(0, 2*pi) for _ in range(nbcols)]
        cols = [complex(cos(a), sin(a)) for a in angles]
        result.append(cols)
    return result

def random_complex_matrices(nbr, nbrows, nbcols):
    """
    Returns a list of matrix of length nbr,
    all of dimension nbrows by nbcols.
    """
    result = []
    for i in range(nbr):
        result.append(random_complex_matrix(nbrows, nbcols))
    return result

def planes_to_string(planes):
    r"""
    Returns one long string with all numbers
    in *planes*, a list of lists of rows.
    The numbers are the real and imaginary parts,
    separated by space.
    """
    result = ""
    for plane in planes:
        for row in plane:
            for cff in row:
                (cffre, cffim) = (cff.real, cff.imag)
                result = result + ' ' + ('%.17e' % cffre)
                result = result + ' ' + ('%.17e' % cffim)
    return result

def points_to_string(pts):
    r"""
    Returns one long string with all numbers in *pts*,
    as sequences of real and imaginary parts,
    every number is separated by one space.
    """
    result = ""
    for row in pts:
        for cff in row:
            (xre, yre) = (cff.real, cff.imag)
            result = result + ' ' + ('%.17e' % xre)
            result = result + ' ' + ('%.17e' % yre)
    return result

def run_pieri_homotopies(mdim, pdim, qdeg, planes, *pts, **opt):
    r"""
    Computes the number of *pdim*-plane producing maps of degree q*deg*
    that meet *mdim*-planes at mdim*pdim + qdeq*(mdim+pdim) points.
    For *qdeg* = 0, there are no interpolation points in *pts*.
    The last parameter in opt contains the value for 'verbose',
    which is True by default.
    """
    from phcpy.phcpy2c2 import py2c_schubert_pieri_count
    from phcpy.phcpy2c2 import py2c_schubert_pieri_homotopies
    from phcpy.phcpy2c2 import py2c_syscon_load_standard_polynomial
    from phcpy.phcpy2c2 import py2c_solcon_number_of_standard_solutions
    from phcpy.phcpy2c2 import py2c_solcon_clear_standard_solutions
    from phcpy.phcpy2c2 import py2c_solcon_length_standard_solution_string
    from phcpy.phcpy2c2 import py2c_solcon_write_standard_solution_string
    if 'verbose' not in opt:
        verbose = True        # by default, the function is verbose
    else:
        verbose = opt['verbose']
    root_count = py2c_schubert_pieri_count(mdim, pdim, qdeg)
    if verbose:
        print 'Pieri root count for', (mdim, pdim, qdeg), 'is', root_count
    strplanes = planes_to_string(planes)
    # print 'length of input data :', len(strplanes)
    if(qdeg > 0):
        strpts = points_to_string(pts[0])
        # print 'the interpolation points :', strpts
    else:
        strpts = ''
    # print 'calling py2c_pieri_homotopies ...'
    py2c_solcon_clear_standard_solutions()
    if verbose:
        print 'passing %d characters for (m, p, q) = (%d, %d, %d)' \
            % (len(strplanes), mdim, pdim, qdeg)
    py2c_schubert_pieri_homotopies(mdim, pdim, qdeg, \
        len(strplanes), strplanes, strpts)
    # print 'making the system ...'
    pols = []
    if(qdeg == 0):
        for i in range(1, mdim*pdim+1):
            pols.append(py2c_syscon_load_standard_polynomial(i))
    else:
        for i in range(1, mdim*pdim+qdeg*(mdim+pdim)+1):
            pols.append(py2c_syscon_load_standard_polynomial(i))
    if verbose:
        print 'the system :'
        for poly in pols:
            print poly
        print 'root count :', root_count
    nbsols = py2c_solcon_number_of_standard_solutions()
    sols = []
    for k in range(1, nbsols+1):
        lns = py2c_solcon_length_standard_solution_string(k)
        sol = py2c_solcon_write_standard_solution_string(k, lns)
        sols.append(sol)
    if verbose:
        print 'the solutions :'
        for solution in sols:
            print solution
    return (pols, sols)

def verify(pols, sols):
    r"""
    Verifies whether the solutions in *sols*
    satisfy the polynomials of the system in *pols*.
    """
    from phcpy.solutions import strsol2dict, evaluate
    dictsols = [strsol2dict(sol) for sol in sols]
    checksum = 0
    for sol in dictsols:
        sumeval = sum(evaluate(pols, sol))
        print sumeval
        checksum = checksum + sumeval
    print 'the total check sum :', checksum

def real_osculating_planes(mdim, pdim, qdeg):
    r"""
    Returns m*p + q*(m+p) real m-planes osculating
    a rational normal curves.
    The values for m, p, and q are provided respectively
    by *mdim*, *pdim*, and *qdeg*.
    """
    from phcpy.phcpy2c2 import py2c_schubert_osculating_planes
    dim = mdim*pdim + qdeg*(mdim+pdim)
    from random import uniform as u
    pts = ""
    for k in range(dim):
        cff = '%.17lf' % u(-1, +1)
        pts = pts + ' ' + cff
    # print 'the points :', pts
    osc = py2c_schubert_osculating_planes(mdim, pdim, qdeg, len(pts), pts)
    # print 'the coefficients of the planes :'
    # print osc
    items = osc.split(' ')
    ind = 0
    planes = []
    for k in range(0, dim):
        plane = []
        for i in range(0, mdim+pdim):
            row = []
            for j in range(0, mdim):
                row.append(eval(items[ind]))
                ind = ind + 1
            plane.append(row)
        planes.append(plane)
    return planes

def make_pieri_system(mdim, pdim, qdeg, planes, is_real=False):
    r"""
    Makes the polynomial system defined by the *mdim*-planes
    in the list *planes*.
    """
    from phcpy.phcpy2c2 import py2c_schubert_pieri_system
    from phcpy.phcpy2c2 import py2c_syscon_load_standard_polynomial
    strplanes = planes_to_string(planes)
    # print 'the string of planes :', strplanes
    if is_real:
        py2c_schubert_pieri_system(mdim, pdim, qdeg, \
            len(strplanes), strplanes, 1)
    else:
        py2c_schubert_pieri_system(mdim, pdim, qdeg, \
            len(strplanes), strplanes, 0)
    result = []
    if(qdeg == 0):
        for i in range(1, mdim*pdim+1):
            result.append(py2c_syscon_load_standard_polynomial(i))
    return result

def cheater(mdim, pdim, qdeg, start, startsols):
    r"""
    Generates a random Pieri problem of dimensions (*mdim, pdim, qdeg*)
    and solves it with a Cheater's homotopy, starting from
    the Pieri system in *start*, at the solutions in *startsols*.
    """
    dim = mdim*pdim + qdeg*(mdim+pdim)
    planes = [random_complex_matrix(mdim+pdim, mdim) for _ in range(0, dim)]
    pols = make_pieri_system(mdim, pdim, qdeg, planes)
    from phcpy.trackers import track
    print 'cheater homotopy with %d paths' % len(startsols)
    sols = track(pols, start, startsols)
    for sol in sols:
        print sol
    verify(pols, sols)

def osculating_input(mdim, pdim, qdeg, start, startsols):
    r"""
    Generates real *mdim*-planes osculating a rational normal curve
    and solves this Pieri problem using the system in *start*,
    with corresponding solutions in *startsols*.
    """
    target_planes = real_osculating_planes(mdim, pdim, qdeg)
    # print 'real osculating planes :', target_planes
    target_system = make_pieri_system(mdim, pdim, qdeg, target_planes, False)
    print 'the start system of length %d :' % len(start)
    for pol in start:
        print pol
    print 'the target system of length %d :' % len(target_system)
    for pol in target_system:
        print pol
    from phcpy.trackers import track
    target_solutions = track(target_system, start, startsols)
    for sol in target_solutions:
        print sol
    verify(target_system, target_solutions)

def test_lrhom(prc='d'):
    """
    Performs a test on the Littlewood-Richardson homotopies.
    """
    brk = [[2, 4, 6], [2, 4, 6], [2, 4, 6]]
    (roco, flags, fsys, sols) \
        = littlewood_richardson_homotopies(6, 3, brk, precision=prc)
    #  outputfilename='/tmp/testlrhomoutput')
    print 'the root count :', roco
    print 'the flags :', flags
    print 'the solutions :'
    for sol in sols:
        print sol
    verify(fsys, sols)

def test_pieri():
    """
    Does a test on the Pieri homotopies.
    """
    (mdim, pdim, qdeg) = prompt_for_dimensions()
    pieri_root_count(mdim, pdim, qdeg)
    dim = mdim*pdim + qdeg*(mdim+pdim)
    planes = [random_complex_matrix(mdim+pdim, mdim) for k in range(0, dim)]
    # print '%d random %d-planes :' % (dim, m)
    # for A in planes:
    #    for row in A: print row
    #    print ''
    if(qdeg > 0):
        points = random_complex_matrix(dim, 1)
        print 'interpolation points :'
        for point in points:
            print point
        (system, sols) = run_pieri_homotopies(mdim, pdim, qdeg, planes, points)
    else:
        (system, sols) = run_pieri_homotopies(mdim, pdim, qdeg, planes)
    print 'evaluation of the solutions :'
    verify(system, sols)
    from phcpy.solver import newton_step
    print 'verification with one Newton step :'
    newton_step(system, sols)
    # cheater(m, p, qdeg, system, sols)
    if(qdeg == 0):
        osculating_input(mdim, pdim, qdeg, system, sols)

def main():
    """
    Tests the Pieri homotopies and the Littlewood-Richardson homotopies.
    """
    print "\nTesting the Pieri homotopies ...\n"
    test_pieri()
    print "\nTesting the Littlewood-Richardson homotopies ..."
    test_lrhom()

if __name__ == "__main__":
    main()



"""
Deflation restores the quadratic convergence of Newton's method
at a singular, isolated solution.
"""
from ctypes import c_int32, c_double, pointer
from phcpy.version import get_phcfun
from phcpy.polynomials import number_of_symbols
from phcpy.polynomials import set_double_system
from phcpy.polynomials import set_double_double_system
from phcpy.polynomials import set_quad_double_system
from phcpy.solutions import set_double_solutions
from phcpy.solutions import get_double_solutions
from phcpy.solutions import set_double_double_solutions
from phcpy.solutions import get_double_double_solutions
from phcpy.solutions import set_quad_double_solutions
from phcpy.solutions import get_quad_double_solutions
from phcpy.solutions import make_solution, diagnostics

def double_newton_step(pols, sols, vrblvl=0):
    r"""
    Applies one Newton step to the *sols* of the *pols*,
    in double precision.
    """
    if vrblvl > 0:
        print('in double_newton_step')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_double_system(nvr, pols, vrblvl-1)
    set_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_newton_step calls phc', end='')
    retval = phc(199, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_solutions(vrblvl-1)
    return result

def double_double_newton_step(pols, sols, vrblvl=0):
    r"""
    Applies one Newton step to the *sols* of the *pols*,
    in double double precision.
    """
    if vrblvl > 0:
        print('in double_double_newton_step')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_double_double_system(nvr, pols, vrblvl-1)
    set_double_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_newton_step calls phc', end='')
    retval = phc(198, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_double_solutions(vrblvl-1)
    return result

def quad_double_newton_step(pols, sols, vrblvl=0):
    r"""
    Applies one Newton step to the *sols* of the *pols*,
    in quad double precision.
    """
    if vrblvl > 0:
        print('in quad_double_newton_step')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_quad_double_system(nvr, pols, vrblvl-1)
    set_quad_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_newton_step calls phc', end='')
    retval = phc(197, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_quad_double_solutions(vrblvl-1)
    return result

def double_deflate(pols, sols, maxitr=3, maxdef=3, \
    tolerr=1.0e-8, tolres=1.0e-8, tolrnk=1.0e-6, vrblvl=0):
    r"""
    Applies deflation on the solutions sols of the system in pols,
    in double precision.  The parameters are as follows:

    *maxitr*: the maximum number of iterations per root,

    *maxdef*: the maximum number of deflations per root,

    *tolerr*: tolerance on the forward error on each root,

    *tolres*: tolerance on the backward error on each root,

    *tolrnk*: tolerance on the numerical rank of the Jacobian matrices.
    """
    if vrblvl > 0:
        print('in double_deflate, maxitr :', maxitr, end='')
        print(', maxdef :', maxdef)
        print(f'tolerr : {tolerr:.3e}', end='')
        print(f', tolres : {tolres:.3e}, tolrnk : {tolrnk:.3e}')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_double_system(nvr, pols, vrblvl-1)
    set_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amaxitr = pointer(c_int32(maxitr))
    bmaxdef = pointer(c_int32(maxdef))
    tols = (c_double * 3)()
    tols[0] = tolerr
    tols[1] = tolres
    tols[2] = tolrnk
    ctols = pointer(tols)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_deflate calls phc', end='')
    retval = phc(196, amaxitr, bmaxdef, ctols, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_solutions(vrblvl-1)
    return result

def double_double_deflate(pols, sols, maxitr=3, maxdef=3, \
    tolerr=1.0e-8, tolres=1.0e-8, tolrnk=1.0e-6, vrblvl=0):
    r"""
    Applies deflation on the solutions sols of the system in pols,
    in double double precision.  The parameters are as follows:

    *maxitr*: the maximum number of iterations per root,

    *maxdef*: the maximum number of deflations per root,

    *tolerr*: tolerance on the forward error on each root,

    *tolres*: tolerance on the backward error on each root,

    *tolrnk*: tolerance on the numerical rank of the Jacobian matrices.
    """
    if vrblvl > 0:
        print('in double_double_deflate, maxitr :', maxitr, end='')
        print(', maxdef :', maxdef)
        print(f'tolerr : {tolerr:.3e}', end='')
        print(f', tolres : {tolres:.3e}, tolrnk : {tolrnk:.3e}')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_double_double_system(nvr, pols, vrblvl-1)
    set_double_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amaxitr = pointer(c_int32(maxitr))
    bmaxdef = pointer(c_int32(maxdef))
    tols = (c_double * 3)()
    tols[0] = tolerr
    tols[1] = tolres
    tols[2] = tolrnk
    ctols = pointer(tols)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_deflate calls phc', end='')
    retval = phc(249, amaxitr, bmaxdef, ctols, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_double_solutions(vrblvl-1)
    return result

def quad_double_deflate(pols, sols, maxitr=3, maxdef=3, \
    tolerr=1.0e-8, tolres=1.0e-8, tolrnk=1.0e-6, vrblvl=0):
    r"""
    Applies deflation on the solutions sols of the system in pols,
    in quad double precision.  The parameters are as follows:

    *maxitr*: the maximum number of iterations per root,

    *maxdef*: the maximum number of deflations per root,

    *tolerr*: tolerance on the forward error on each root,

    *tolres*: tolerance on the backward error on each root,

    *tolrnk*: tolerance on the numerical rank of the Jacobian matrices.
    """
    if vrblvl > 0:
        print('in quad_double_deflate, maxitr :', maxitr, end='')
        print(', maxdef :', maxdef)
        print(f'tolerr : {tolerr:.3e}', end='')
        print(f', tolres : {tolres:.3e}, tolrnk : {tolrnk:.3e}')
        print('the polynomial system :')
        for pol in pols:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    nvr = number_of_symbols(pols, vrblvl-1)
    set_quad_double_system(nvr, pols, vrblvl-1)
    set_quad_double_solutions(nvr, sols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amaxitr = pointer(c_int32(maxitr))
    bmaxdef = pointer(c_int32(maxdef))
    tols = (c_double * 3)()
    tols[0] = tolerr
    tols[1] = tolres
    tols[2] = tolrnk
    ctols = pointer(tols)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_deflate calls phc', end='')
    retval = phc(250, amaxitr, bmaxdef, ctols, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_quad_double_solutions(vrblvl-1)
    return result

def double_multiplicity(system, solution, \
    order=5, tol=1.0e-8, vrblvl=0):
    r"""
    Computes the multiplicity structure in double precision
    of an isolated solution (in the string *solution*)
    of a polynomial system (in the list *system*).
    The other parameters are

    *order*: the maximum order of differentiation,

    *tol*: tolerance on the numerical rank,

    *vrblvl*: is the verbose level.

    On return is the computed multiplicity.
    """
    if vrblvl > 0:
        print('in double_multiplicity ...')
        print('the polynomial system :')
        for pol in system:
            print(pol)
        print('the solution :')
        print(solution)
    dim = number_of_symbols(system, vrblvl-1)
    set_double_system(dim, system, vrblvl-1)
    set_double_solutions(dim, [solution], vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amlt = pointer(c_int32(order))
    size = order + 1
    bhlb = (c_int32 * size)()
    bhlb[0] = vrblvl
    phlb = pointer(bhlb)
    ctol = pointer(c_double(tol))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_multiplicity calls phc', end='')
    retval = phc(732, amlt, phlb, ctol, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    vals = phlb[0:size]
    hlbfun = []
    for idx in range(size):
        hlbfun.append(int(vals[0][idx]))
    if vrblvl > 0:
        print('The values of the Hilbert function :', hlbfun)
        print('The multiplicity :', amlt[0])
    return (amlt[0], hlbfun)

def double_double_multiplicity(system, solution, \
    order=5, tol=1.0e-8, vrblvl=0):
    r"""
    Computes the multiplicity structure in double double precision
    of an isolated solution (in the string *solution*)
    of a polynomial system (in the list *system*).
    The other parameters are

    *order*: the maximum order of differentiation,

    *tol*: tolerance on the numerical rank,

    *vrblvl*: is the verbose level.

    On return is the computed multiplicity.
    """
    if vrblvl > 0:
        print('in double_double_multiplicity ...')
        print('the polynomial system :')
        for pol in system:
            print(pol)
        print('the solution :')
        print(solution)
    dim = number_of_symbols(system, vrblvl-1)
    set_double_double_system(dim, system, vrblvl-1)
    set_double_double_solutions(dim, [solution], vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amlt = pointer(c_int32(order))
    size = order + 1
    bhlb = (c_int32 * size)()
    bhlb[0] = vrblvl
    phlb = pointer(bhlb)
    ctol = pointer(c_double(tol))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> double_double_multiplicity calls phc', end='')
    retval = phc(733, amlt, phlb, ctol, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    vals = phlb[0:size]
    hlbfun = []
    for idx in range(size):
        hlbfun.append(int(vals[0][idx]))
    if vrblvl > 0:
        print('The values of the Hilbert function :', hlbfun)
        print('The multiplicity :', amlt[0])
    return (amlt[0], hlbfun)

def quad_double_multiplicity(system, solution, \
    order=5, tol=1.0e-8, vrblvl=0):
    r"""
    Computes the multiplicity structure in quad double precision
    of an isolated solution (in the string *solution*)
    of a polynomial system (in the list *system*).
    The other parameters are

    *order*: the maximum order of differentiation,

    *tol*: tolerance on the numerical rank,

    *vrblvl*: is the verbose level.

    On return is the computed multiplicity.
    """
    if vrblvl > 0:
        print('in quad_double_multiplicity ...')
        print('the polynomial system :')
        for pol in system:
            print(pol)
        print('the solution :')
        print(solution)
    dim = number_of_symbols(system, vrblvl-1)
    set_quad_double_system(dim, system, vrblvl-1)
    set_quad_double_solutions(dim, [solution], vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    amlt = pointer(c_int32(order))
    size = order + 1
    bhlb = (c_int32 * size)()
    bhlb[0] = vrblvl
    phlb = pointer(bhlb)
    ctol = pointer(c_double(tol))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> quad_double_multiplicity calls phc', end='')
    retval = phc(734, amlt, phlb, ctol, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    vals = phlb[0:size]
    hlbfun = []
    for idx in range(size):
        hlbfun.append(int(vals[0][idx]))
    if vrblvl > 0:
        print('The values of the Hilbert function :', hlbfun)
        print('The multiplicity :', amlt[0])
    return (amlt[0], hlbfun)

def test_double_newton_step(vrblvl=0):
    """
    Tests Newton's method in double precision.
    """
    if vrblvl > 0:
        print('in test_double_newton_step ...')
    pols = ['x^2 - 1;', 'x*y - 1;']
    sol = make_solution(['x', 'y'], [0.9999, 0.9999])
    sols = [sol]
    newsols = double_newton_step(pols, sols, vrblvl)
    if vrblvl > 0:
        print('solution after a first Newton step :')
        print(newsols[0])
    newsols = double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a second Newton step :')
        print(newsols[0])
    err, rco, res = diagnostics(newsols[0], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-12)

def test_double_double_newton_step(vrblvl=0):
    """
    Tests Newton's method in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_newton_step ...')
    pols = ['x^2 - 1;', 'x*y - 1;']
    sol = make_solution(['x', 'y'], [0.9999, 0.9999])
    sols = [sol]
    newsols = double_double_newton_step(pols, sols, vrblvl)
    if vrblvl > 0:
        print('solution after a first Newton step :')
        print(newsols[0])
    newsols = double_double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a second Newton step :')
        print(newsols[0])
    newsols = double_double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a third Newton step :')
        print(newsols[0])
    err, rco, res = diagnostics(newsols[0], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-24)

def test_quad_double_newton_step(vrblvl=0):
    """
    Tests Newton's method in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_newton_step ...')
    pols = ['x^2 - 1;', 'x*y - 1;']
    sol = make_solution(['x', 'y'], [0.9999, 0.9999])
    sols = [sol]
    newsols = quad_double_newton_step(pols, sols, vrblvl)
    if vrblvl > 0:
        print('solution after a first Newton step :')
        print(newsols[0])
    newsols = quad_double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a second Newton step :')
        print(newsols[0])
    newsols = quad_double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a third Newton step :')
        print(newsols[0])
    newsols = quad_double_newton_step(pols, newsols, vrblvl)
    if vrblvl > 0:
        print('solution after a fourth Newton step :')
        print(newsols[0])
    err, rco, res = diagnostics(newsols[0], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-48)

def test_double_deflate(vrblvl=0):
    """
    Tests deflation in double precision,
    on the 'ojika1' demonstration test system, in the paper by
    T. Ojika: "Modified deflation algorithm for the solution of 
    singular problems. I. A system of nonlinear algebraic equations."
    J. Math. Anal. Appl. 123, 199-221, 1987.
    """
    pols = ['x^2 + y - 3;', 'x + 0.125*y^2 - 1.5;']
    regsol = make_solution(['x', 'y'], [-2.999, -5.999]) # regular
    sinsol = make_solution(['x', 'y'], [0.9999, 1.999])  # singular
    sols = [regsol, sinsol]
    defsols = double_deflate(pols, sols, vrblvl=vrblvl)
    print('the first solution after deflation :')
    print(defsols[0])
    print('the second solution after deflation :')
    print(defsols[1])
    err, rco, res = diagnostics(defsols[1], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-12)

def test_double_double_deflate(vrblvl=0):
    """
    Tests deflation in double double precision,
    on the 'ojika1' demonstration test system, in the paper by
    T. Ojika: "Modified deflation algorithm for the solution of 
    singular problems. I. A system of nonlinear algebraic equations."
    J. Math. Anal. Appl. 123, 199-221, 1987.
    """
    pols = ['x^2 + y - 3;', 'x + 0.125*y^2 - 1.5;']
    regsol = make_solution(['x', 'y'], [-2.999, -5.999]) # regular
    sinsol = make_solution(['x', 'y'], [0.9999, 1.999])  # singular
    sols = [regsol, sinsol]
    defsols = double_double_deflate(pols, sols, vrblvl=vrblvl)
    print('the first solution after deflation :')
    print(defsols[0])
    print('the second solution after deflation :')
    print(defsols[1])
    err, rco, res = diagnostics(defsols[1], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-12)

def test_quad_double_deflate(vrblvl=0):
    """
    Tests deflation in quad double precision,
    on the 'ojika1' demonstration test system, in the paper by
    T. Ojika: "Modified deflation algorithm for the solution of 
    singular problems. I. A system of nonlinear algebraic equations."
    J. Math. Anal. Appl. 123, 199-221, 1987.
    """
    pols = ['x^2 + y - 3;', 'x + 0.125*y^2 - 1.5;']
    regsol = make_solution(['x', 'y'], [-2.999, -5.999]) # regular
    sinsol = make_solution(['x', 'y'], [0.9999, 1.999])  # singular
    sols = [regsol, sinsol]
    defsols = quad_double_deflate(pols, sols, vrblvl=vrblvl)
    print('the first solution after deflation :')
    print(defsols[0])
    print('the second solution after deflation :')
    print(defsols[1])
    err, rco, res = diagnostics(defsols[1], vrblvl-1)
    if vrblvl > 0:
        print(f'forward error : {err:.3e}')
        print(f'estimate for inverse condition number : {rco:.3e}')
        print(f'backward error : {res:.3e}')
    return int(res > 1.0e-12)

def test_double_multiplicity(vrblvl=0):
    """
    Tests the multiplicity structure in double precision.
    """
    if vrblvl > 0:
        print('in test_double_multiplicity ...')
    pols = [ 'x**2+y-3;', 'x+0.125*y**2-1.5;']
    sol = make_solution(['x', 'y'], [1, 2])
    multiplicity, hilfun = double_multiplicity(pols, sol, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the multiplicity :', multiplicity)
        print('the Hibert function :', hilfun)
    return int(multiplicity != 3)

def test_double_double_multiplicity(vrblvl=0):
    """
    Tests the multiplicity structure in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_multiplicity ...')
    pols = [ 'x**2+y-3;', 'x+0.125*y**2-1.5;']
    sol = make_solution(['x', 'y'], [1, 2])
    multiplicity, hilfun = double_double_multiplicity(pols, sol, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('the multiplicity :', multiplicity)
        print('the Hibert function :', hilfun)
    return int(multiplicity != 3)

def test_quad_double_multiplicity(vrblvl=0):
    """
    Tests the multiplicity structure in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_multiplicity ...')
    pols = [ 'x**2+y-3;', 'x+0.125*y**2-1.5;']
    sol = make_solution(['x', 'y'], [1, 2])
    multiplicity, hilfun = quad_double_multiplicity(pols, sol, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('the multiplicity :', multiplicity)
        print('the Hibert function :', hilfun)
    return int(multiplicity != 3)

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_double_newton_step(lvl)
    fail = fail + test_double_double_newton_step(lvl)
    fail = fail + test_quad_double_newton_step(lvl)
    fail = fail + test_double_deflate(lvl)
    fail = fail + test_double_double_deflate(lvl)
    fail = fail + test_quad_double_deflate(lvl)
    fail = fail + test_double_multiplicity(lvl)
    fail = fail + test_double_double_multiplicity(lvl)
    fail = fail + test_quad_double_multiplicity(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()



#!/usr/bin/python

# Enter debug mode
from phc_config import phc_debug, style_forgetpwd
if phc_debug:
   import cgitb; cgitb.enable()

import cgi, Cookie, os, sha, datetime, time

from phc_html import PrintHeader, Footer_Bar
from phc_sql import sql_forgetpwd

"""
infos[0]: Email
infos[1]: password
infos[2]: repeated password
infos[3]: Name_First
infos[4]: Name_Last
infos[5]: Org

error[0]: 1. general error
          2. password don't match
error[1]: infos[0]: Email
error[2]: infos[1]: password
error[3]: infos[2]: repeated password
error[4]: infos[3]: Name_First
error[5]: infos[4]: Name_Last
error[6]: infos[5]: Org
"""

def ProcessName(form):
   """
   Processes name of Email form.
   Returns True if error, else False.
   """
   error = [0,0,0,0,0]
   infos = ["","","",""]
   form_list = ['login','Name_First','Name_Last','Org']

   for i in xrange(len(form_list)):
      try:
         infos[i] = form[form_list[i]].value
      except KeyError:
         #print "Please enter your email"
         error[i+1] = 1
         error[0] = 1

   return infos, error

def handle_error(error):
   if error[0] == 1:
      print "Please enter these information"
      info = ["Email", "First Name", "Last Name", "Organization" ]
      for i in xrange(0, len(info)):
         if error[i+1]==1:
            print ", " + info[i]
   elif error[0] == 2:
      print "Sorry. Your password don't match."
   elif error[0] == 3:
      print """Sorry. Your email has been registered. 
         <a href='Find_Password.html'>Find my password</a>"""
   elif error[0] == 4:
      print """Sorry. We couldn't send the email to your mailbox. Your information doesn't match our database. Please check your email address and other information."""

def main():
   """
   Form to process Email.
   """
   PrintHeader(style_forgetpwd)
   form = cgi.FieldStorage()
   infos, error = ProcessName(form)
   
   Email_Address = infos[0]
   Name_First    = infos[1]
   Name_Last     = infos[2]
   Org           = infos[3]
   
   if not error[0]:
      error[0] = sql_forgetpwd(Email_Address, Name_First, Name_Last, Org)
      if not error[0]:
         print """
               The reset link has been sent to your mailbox.</p>
               Please check your mailbox to reset your password."""
         
   if error[0]:
      print """<html><body>
               <div>
               <h1> Forget password </h1>
               <form method = "post" 
                     action = "forgetpwd.py">"""
      print """<table border="0">"""

      # Email Address
      print """<tr><td class="reg_form">Email</td>"""

      if Email_Address == '':
         print """<td><input type="text" name="login" spellcheck="false">
         </td><td></td>"""
      else:
         if error[0] == 4:
            print """<td><input type = "text" name = "login"  
                                 spellcheck="false"  value =%s ></td><td><font color="red">  X</font></td>"""% infos[0]
         else:
            print """<td><input type = "text" name = "login"  
                                 spellcheck="false"  value =%s></td><td>V</td>"""% Email_Address

      print "</tr>"

      form_list = [[Name_First, 'First Name', 'Name_First'],\
                   [Name_Last , 'Last Name','Name_Last'],\
                   [Org, 'Organization','Org']]
      
      # First Name
      for i in xrange(len(form_list)):
         print """<tr><td class="reg_form">%s</td>"""%form_list[i][1]
         if form_list[i][0] == '':
            print """<td><input type="text" name="%s" spellcheck="false" >
            </td></tr>"""%form_list[i][2]
         else:
            print """<td><input type = "text" name = "%s"  
                   spellcheck="false"  value =%s></td><td>V</td>"""\
                  %(form_list[i][2],form_list[i][0])
         print "</tr>"

      print "</table>"

      # Submit bottum
      print """<input type="submit" value="Submit">
         </form>
         </div>"""
   handle_error(error)
   Footer_Bar()
   print "</body></html>\n"

main()




"""
The module families contains scripts to generate polynomial systems
for any dimension.
"""

def cyclic(dim):
    """
    Returns a list of string representing the polynomials
    of the cyclic n-roots system.
    This system entered the computer algebra literature in a technical
    report by J. Davenport on Looking at a set of equations,
    published in 1987 as Bath Computer Science Technical Report 87-06.
    Another technical report by J. Backelin in 1989 has the title
    Square multiples n give infinitely many cyclic n-roots, publised as
    Reports, Matematiska Institutionen 8, Stockholms universitet.
    Another interesting preprint is written by U. Haagerup,
    available at http://www.math.ku.dk/~haagerup, on cyclic p-roots
    of prime length p and related complex Hadamard matrices.
    """
    result = []
    for i in range(dim-1):
        pol = ''
        for j in range(dim):
            term = 'x' + str(j)
            for k in range(i):
                term = term + '*' + 'x' + str((j+k+1) % dim)
            if(j == 0):
                pol = term
            else:
                pol = pol + ' + ' + term
        result.append(pol + ';')
    pol = 'x0'
    for i in range(1, dim):
        pol = pol + '*x' + str(i)
    pol = pol + ' - 1;'
    result.append(pol)
    return result

def katsura_variable(var, dim):
    """
    Returns the variable U(var, dim) for use in the function katsura.
    """
    if(var > dim or var < -dim):
        return ''
    elif(var < 0):
        return 'u' + str(-var)
    else:
        return 'u' + str(var)

def katsura(dim):
    """
    Returns the list of strings to represent the system of Katsura.
    The system originated in a paper by S. Katsura on
    Spin glass problem by the method of integral equation
    of the effective field, published in 1990 by World Scientific
    in the volume New Trends in Magnetism, edited
    by M. D. Coutinho-Filho and S. M. Resende, pages 110-121.
    The note by S. Katsura on Users posing problems to PoSSo appeared
    in 1994 in the second number of the PoSSo Newsletter,
    edited by L. Gonzalez-Vega and T. Recio.
    """
    result = []
    pol = katsura_variable(-dim, dim)
    for i in range(-dim+1, dim+1):
        pol = pol + ' + ' + katsura_variable(i, dim)
    pol = pol + ' - 1'
    result.append(pol + ';')
    for ind in range(dim):
        pol = katsura_variable(-dim, dim) + '*' + katsura_variable(dim, dim)
        for i in range(-dim+1, dim+1):
            pol = pol + ' + ' + katsura_variable(i, dim)
            var = katsura_variable(ind-i, dim)
            if(var != ''):
                pol = pol +  '*' + katsura_variable(ind-i, dim)
        pol = pol + ' - ' + katsura_variable(ind, dim)
        result.append(pol + ';')
    return result

def noon(dim, parameter=1.1):
    """
    Returns the list of strings to represent the system of Noonburg.
    The system originates in a paper by V. W. Noonburg on
    a neural network modeled by an adaptive Lotka-Volterra system,
    published in 1989 in volume 49 of SIAM Journal on Applied Mathematics,
    pages 1779-1792.
    It appeared also in a paper by K. Gatermann entitled Symbolic solution
    of polynomial equation systems with symmetry, published by ACM in 1990
    in the proceedings of ISSAC-90, pages 112-119.
    """
    result = []
    for i in range(dim):
        pol = 'x' + str(i+1) + '*('
        for j in range(dim):
            if(i != j):
                if(pol[-1] != '('):
                    pol = pol + ' + '
                pol = pol + 'x' + str(j+1) + '^2'
        pol = pol + ') - ' + str(parameter) + '*x' + str(i+1)
        pol = pol + ' + 1;'
        result.append(pol)
    return result

def indeterminate_matrix(rows, cols):
    """
    Returns a list of lists with as many lists
    as the value of rows.  Each rows has as many
    indeterminates as the value of cols.
    The lists of lists contains the data for a
    matrix of dimension rows by cols of variables.
    """
    result = []
    for i in range(0, rows):
        ithrow = []
        for j in range(0, cols):
            name = 'x' + '_' + str(i+1) + '_' + str(j+1)
            ithrow.append(name)
        result.append(ithrow)
    return result

def adjacent_minors(rows, cols):
    """
    Returns all adjacent 2-by-2 minors of a general
    matrix of dimensions rows by cols.
    This system originated in a paper on lattice walks and
    primary decomposition, written by P. Diaconis, D. Eisenbud,
    and B. Sturmfels, published by Birkhauser in 1998 in
    Mathematical Essays in Honor of Gian-Carlo Rota,
    edited by B. E. Sagan and R. P. Stanley,
    volume 161 of Progress in Mathematics, pages 173--193.
    See also the paper by S. Hosten and J. Shapiro on
    Primary decomposition of lattice basis ideals, published in 2000
    in the Journal of Symbolic Computation, volume 29, pages 625-639.
    """
    vrs = indeterminate_matrix(rows, cols)
    result = []
    for i in range(0, rows-1):
        for j in range(0, cols-1):
            equ = vrs[i][j] + '*' + vrs[i+1][j+1]
            equ = equ +  '-'
            equ = equ + vrs[i+1][j] + '*' + vrs[i][j+1]
            result.append(equ + ';')
    return result

def pieri_problem(mdim, pdim, real=True):
    """
    Returns a system that expresses the intersection of pdim-planes
    with mdim*pdim general mdim-planes in (mdim+pdim)-space.
    When real is True, the generated mdim-planes are osculating
    a rational normal curve and all solutions are expected to be real.
    If real is False, then random complex planes are generated.
    For reality of solutions of polynomial systems, see the book by
    Frank Sottile: Real Solutions to Equations from Geometry,
    volume 57 of University Lecture Series, AMS, 2011.
    """
    from schubert import make_pieri_system
    from schubert import real_osculating_planes
    from schubert import random_complex_matrices
    if real:
        planes = real_osculating_planes(mdim, pdim, 0)
    else:
        planes = random_complex_matrices(mdim*pdim, mdim, pdim)
    return make_pieri_system(mdim, pdim, 0, planes)

def recpol(nbplayers, player, ind, acc):
    """
    Recursive generation of one polynomial,
    called by the function nash below.
    """
    from random import uniform
    result = ''
    prob = 'p' + str(ind)
    if(ind != player):
        (r1u, r2u) = (uniform(0,1), uniform(0,1))
        if(acc == '1'):
            result += ' + ' + str(r1u) + '*' + prob + ' + ' \
                   + str(r2u) + '*(1 - ' + prob + ')'
        else:
            result += ' + ' + str(r1u) + '*' + prob + '*' + acc + ' + ' \
                   + str(r2u) + '*(1 - ' + prob + ')*' + acc 
        if(ind < nbplayers):
            if(acc == '1'):
                result += recpol(nbplayers, player, ind+1, prob) \
                       + recpol(nbplayers, player, ind+1, \
                                '(1 - ' + prob + ')')
            else:
                result += recpol(nbplayers, player, ind+1, \
                                 prob + '*' + acc) \
                       + recpol(nbplayers, player, ind+1, \
                                '(1 - ' + prob + ')*' + acc)
    elif(ind < nbplayers):
        result = recpol(nbplayers, player, ind+1, acc)
    return result

def nash(nbplayers, player):
    """
    Returns the string representation of one equation for a player 
    to compute the totally mixed Nash equibria for nbplayers with 
    two pure strategies, with random positive utilities.
    """
    equ = recpol(nbplayers, player, 1, '1')
    return equ[2:] + ';' # bite off leading ' +'

def generic_nash_system(nbplayers):
    """
    Returns a list of strings representing polynomials that
    define totally mixed Nash equilibria for a number of players
    equals to nbplayers with two pure strategies.
    The problem setup is generic in the sense that the utilities
    are uniformly generated positive floats in [0,1].
    For n players, the n-homogeneous Bezout number provides
    a generically exact count on the number of equilibria, 
    see the paper by Richard D. McKelvey and Andrew McLennan on
    the maximal number of regular totally mixed Nash equilibria,
    published in the Journal of Economic Theory, volume 72,
    pages 411-425, 1997.
    """
    result = [nash(nbplayers,k) for k in range(1,nbplayers+1)]
    return result

def chandra(dim, par=0.51234):
    """
    Generates the equations of the Chandrasekhar H-equation
    for the given dimension dim and parameter par.
    The name of the problem stems from the 1960 Dover publication
    Radiative Transfer by S. Chandrasekhar.  The problem was used
    as an illustration in the paper by C.T. Kelley on Solution of
    the Chandrasekhar H-equation by Newton's method, published in
    J. Math. Phys. 21, pages 1625-1628, 1980.  It featured in 
    the paper by Jorge J. More on A collection of nonlinear model
    problems, published in volume 26 of the Lectures in Applied
    Mathematics, pages 723-762, AMS 1990 and in the paper by
    Laureano Gonzalez-Vega on Some examples on problem solving by
    using the symbolic viewpoint when dealing with polynomial
    systems of equations, published in Computer Algebra in Science
    and Engineering, pages 102-116, World Scientific, 1995.
    """
    result = []
    cff = str(2*dim)
    for k in range(1, dim+1):
        equ = cff + '*H' + str(k) + ' - ' + str(par) + '*H' + str(k) + '*(1'
        for i in range(1,dim):
            equ = equ + '+(' + str(k) + '/' + str(i+k) + str(')*H') + str(i)
        equ = equ + ') - ' + cff + ';' 
        result.append(equ)
    return result

def strvar(name, i, j):
    """
    Returns the string representation for the variable with the given name
    and indices i and j, i != j.  Swaps the values for i and j if i > j.
    """
    result = name + '_'
    if(i < j):
        result = result + str(i) + '_' + str(j)
    else:
        result = result + str(j) + '_' + str(i)
    return result

def firsteqs(dim):
    """
    Returns the list of equations defining the relations between the
    S[i,j] and the r[i,j] variables, for all i < j, for i from 1 to dim-1.
    Since the S[i,j] variables occur linearly, with these equations we
    can rewrite S[i,j] in terms of the corresponding r[i,j] variables.
    The elimination of the S[i,j] comes at the expense of high degrees
    in the r[i,j] variables of the remaining equations.
    """
    result = []
    for i in range(1, dim):
        for j in range(i+1, dim+1):
            equ = strvar('r', i, j) + '**3*('
            equ = equ + strvar('S', i, j) + ' + 1) - 1;'
            result.append(equ)
    return result

def poleqs(dim, masses):
    """
    Returns the list of polynomial equations for the central configurations,
    for as many masses as the dimension dim.
    """
    result = []
    for i in range(1, dim):
        for j in range(i+1, dim+1):
            equ = ''
            for k in range(1, dim+1):
                if(k > 1):
                    equ = equ + ' + '
                equ = equ + str(masses[k-1]) + '*('
                if(k != i):
                    equ = equ + strvar('S', i, k) + '*('
                    if(k != j):
                        equ = equ + strvar('r', j, k) + '**2'
                    equ = equ + ' - ' + strvar('r', i, k) + '**2'
                    equ = equ + ' - ' + strvar('r', i, j) + '**2)'
                if(k != j):
                    if(equ[-1] != '('):
                        equ = equ + ' + '
                    equ = equ + strvar('S', j, k) + '*('
                    if(k != i):
                        equ = equ + strvar('r', i, k) + '**2'
                    equ = equ + ' - ' + strvar('r', j, k) + '**2'
                    equ = equ + ' - ' + strvar('r', i, j) + '**2)'
                equ = equ + ')'
            equ = equ + ';'
            result.append(equ)
    return result

def nbodyeqs(dim, mas):
    """
    The central configurations of the n-body problem can be defined via the
    Albouy-Chenciner equations, by A. Albouy and A. Chenciner: Le probleme
    des n corps et les distances mutuelles. Inv. Math. 131, 151-184, 1998;
    and the paper by M. Hampton and R. Moeckel on Finiteness of relative 
    equilibria of the four-body problem. Inv. Math. 163, 289-312, 2006.
    Returns a list of strings, representing the central configurations
    for the n-body problem, where n = dim and with masses in the list mas.
    We require that len(mas) == dim.
    """
    result = firsteqs(dim)
    polsys = poleqs(dim, mas)
    for pol in polsys:
        result.append(pol)
    return result

def test():
    """
    Writes particular instances of the systems in the families.
    """
    print '\ncyclic 5-roots :\n'
    for pol in cyclic(5):
        print pol
    print '\nnoon for n = 5 :\n'
    for pol in noon(5):
        print pol
    print '\nkatsura for n = 5 :\n'
    for pol in katsura(5):
        print pol
    print '\nadjacent 2-by-2 minors of a 3-by-5 matrix :\n'
    for pol in adjacent_minors(3, 5):
        print pol
    print '\nintersection of 6 general 3-planes with 2-planes :\n'
    for pol in pieri_problem(3, 2):
        print pol
    print '\nNash equilibria for game with 4 players and 2 strategies :\n'
    for pol in generic_nash_system(4):
        print pol
    print '\nChandrasekhar H-equation for dimension 4 :\n'
    for pol in chandra(4):
        print pol
    print '\nCentral configurations for 3 equal masses :\n'
    for pol in nbodyeqs(3, [1, 1, 1]):
        print pol

if __name__ == "__main__":
    test()



"""
A an artificial parameter homotopy is a family of polynomial systems 
which connects a given target system to a start system.
The module starters exports several functions to construct start systems.
Analoguous to the culinary starters, start systems may be viewed as
systems that are easier to solve (or digest) than general systems.
"""
from ctypes import c_int32, c_double, pointer, create_string_buffer
from phcpy.version import int4a2str, str2int4a
from phcpy.version import get_phcfun
from phcpy.polynomials import set_double_system, string_of_symbols
from phcpy.polynomials import get_double_system
from phcpy.polynomials import degree_of_double_polynomial
from phcpy.solutions import get_double_solutions
from phcpy.examples import noon3, game4two
from phcpy.solver import solve_checkin, solve

def total_degree(pols, vrblvl=0):
    """
    Given in pols a list of string representations of polynomials,
    returns the product of the degrees of the polynomials,
    the so-called total degree which bounds the number of
    isolated solutions of the polynomial system.
    The system is assumed to be square.
    The value of the verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in total degree, pols :')
        for pol in pols:
            print(pol)
    set_double_system(len(pols), pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    deg = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> total_degree calls phc', end='')
    retval = phc(28, deg, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('the total degree :', deg[0])
    return deg[0]

def total_degree_start_system(pols, checkin=True, vrblvl=0):
    r"""
    Returns the system and solutions of the total degree start system
    for the polynomials represented by the strings in the list *pols*.
    If *checkin*, then the list *pols* is tested to see if *pols* defines
    a square polynomial system.  If the input system is not square,
    then an error message is printed and None is returned.
    """
    if vrblvl > 0:
        print('in total degree, pols :')
        for pol in pols:
            print(pol)
    if checkin:
        errmsg = 'Start systems are defined only for square systems,'
        if not solve_checkin(pols, errmsg):
            return None
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    svars = string_of_symbols(200, vrblvl-1)
    degrees = [degree_of_double_polynomial(k+1) for k in range(dim)]
    result = []
    for ind in range(dim):
        result.append(svars[ind]+'^'+str(degrees[ind])+' - 1;')
    return (result, solve(result))

def m_homogeneous_bezout_number(pols, vrblvl=0):
    r"""
    Given in *pols* a list of string representations of polynomials,
    in as many variables as the elements in the list,
    this function applies a heuristic to generate a partition of the
    set of unknowns to exploit the product structure of the system.
    On return are the m-homogeneous Bezout number and the partition
    of the set of unknowns.  If the partition equals the entire
    set of unknowns, then the 1-homogeneous Bezout number equals
    the total degree of the system.
    """
    if vrblvl > 0:
        print('in m_homogeneous_bezout_number, pols :')
        for pol in pols:
            print(pol)
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    deg = pointer(c_int32(0))
    pbuffer = create_string_buffer(b"", 4*256)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> m_homogeneous_bezout_number calls phc', end='')
    retval = phc(530, deg, pbuffer, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('an m-homogeneous Bezout number :', deg[0])
    wholepartition = int4a2str(pbuffer, vrblvl=vrblvl-1)
    endidx = wholepartition.find('\0')
    partition = wholepartition[:endidx]
    if vrblvl > 0:
        print('the partition :', partition)
    return (deg[0], partition)

def m_partition_bezout_number(pols, partition, vrblvl=0):
    r"""
    There are as many m-homogeneous Bezout numbers as there are
    partitions of the set of unknowns of a polynomial system.
    Given in *pols* the string representations of a polynomial system
    in as many variables as equations, and a string representation of
    a *partition* of the set of unknowns, this function returns the
    m-homogeneous Bezout number corresponding to the given partition.
    """
    if vrblvl > 0:
        print('in m_partition_bezout_number, pols :')
        for pol in pols:
            print(pol)
        print('the partition :', partition)
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    deg = pointer(c_int32(len(partition)))
    pbuffer = str2int4a(partition, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> m_partition_bezout_number calls phc', end='')
    retval = phc(531, deg, pbuffer, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('the m-homogeneous Bezout number :', deg[0])
    return deg[0]

def m_homogeneous_start_system(pols, partition, checkin=True, vrblvl=0):
    r"""
    For an m-homogeneous Bezout number of a polynomial system defined by
    a *partition* of the set of unknowns, one can define a linear-product
    system that has exactly as many regular solutions as the Bezount number.
    This linear-product system can then be used as start system in a
    homotopy to compute all isolated solutions of any polynomial system
    with the same m-homogeneous structure.
    This function returns a linear-product start system with random
    coefficients and its solutions for the given polynomials in pols
    and the partition.
    If *checkin*, then the list *pols* is tested to see if *pols* defines
    a square polynomial system.  If the input system is not square,
    then an error message is printed and None is returned.
    """
    if vrblvl > 0:
        print('in m_homogeneous_start_system, pols :')
        for pol in pols:
            print(pol)
        print('the partition :', partition)
    if checkin:
        errmsg = 'Start systems are defined only for square systems,'
        if not solve_checkin(pols, errmsg):
            return None
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    deg = pointer(c_int32(len(partition)))
    pbuffer = str2int4a(partition, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> m_partition_start_system calls phc', end='')
    retval = phc(532, deg, pbuffer, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('the m-homogeneous Bezout number :', deg[0])
    startsys = get_double_system(vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in startsys:
            print(pol)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> m_partition_start_system calls phc', end='')
        print(' to solve system')
    retval = phc(114, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    startsols = get_double_solutions(vrblvl-1)
    if vrblvl > 0:
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('solution', idx+1, ':')
            print(sol)
    return (startsys, startsols)

def linear_product_root_count(pols, checkin=True, vrblvl=0):
    r"""
    Given in *pols* a list of string representations of polynomials,
    returns a linear-product root count based on a supporting
    set structure of the polynomials in *pols*.  This root count is
    an upper bound for the number of isolated solutions.
    """
    if vrblvl > 0:
        print('in linear_product_root_count ...')
        print('the polynomials :')
        for pol in pols:
            print(pol)
    if checkin:
        errmsg = 'Root counts are defined only for square systems,'
        if not solve_checkin(pols, errmsg):
            return None
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    roco = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> linear_product_root_count calls phc', end='')
    retval = phc(110, roco, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> linear_product_root_count calls phc', end='')
    retval = phc(112, roco, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('linear product root count :', roco[0])
    lprc = roco[0]
    strsets = create_string_buffer(b"", 4*1024)
    if vrblvl > 0:
        print('-> linear_product_root_count calls phc', end='')
    retval = phc(116, roco, strsets, ccc, vrb)
    wholesets = int4a2str(strsets, vrblvl=vrblvl-1)
    endidx = wholesets.find('\0')
    sets = wholesets[:endidx]
    if vrblvl > 0:
        print(', return value :', retval)
        print('supporting set structure :')
        print(sets)
    return (lprc, sets)

def random_linear_product_system(pols, checkin=True, tosolve=True, vrblvl=0):
    r"""
    Given in *pols* a list of string representations of polynomials,
    returns a random linear-product system based on a supporting
    set structure and its solutions as well (if *tosolve*).
    If *checkin*, then the list *pols* is tested to see if *pols* defines
    a square polynomial system.  If the input system is not square,
    then an error message is printed and None is returned.
    """
    if vrblvl > 0:
        print('in random_linear_product_system ...')
        print('the polynomials :')
        for pol in pols:
            print(pol)
    if checkin:
        errmsg = 'Root counts are defined only for square systems,'
        if not solve_checkin(pols, errmsg):
            return None
    dim = len(pols)
    set_double_system(dim, pols, vrblvl-1)
    phc = get_phcfun(vrblvl-1)
    roco = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> random_linear_product_system calls phc', end='')
    retval = phc(110, roco, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    if vrblvl > 0:
        print('-> random_linear_product_system calls phc', end='')
    retval = phc(113, roco, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = get_double_system()
    if not tosolve:
        return result
    if vrblvl > 0:
        print('-> random_linear_product_system calls phc', end='')
    retval = phc(114, roco, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    sols = get_double_solutions()
    return (result, sols)

def test_total_degree(vrblvl=0):
    """
    Tests the total degree and the start system.
    """
    if vrblvl > 0:
        print('in test_total_degree ...')
    pols = noon3()
    totdeg = total_degree(pols, vrblvl)
    if vrblvl > 0:
        print('the total degree of noon3 :', totdeg)
    start, startsols = total_degree_start_system(pols, vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    return int(len(startsols) != 27)

def test_m_homogeneous_degree(vrblvl=0):
    """
    Tests m-homogeneous Bezout number.
    """
    if vrblvl > 0:
        print('in test_m_homogeneous_degree ...')
    pols = game4two()
    deg, partition = m_homogeneous_bezout_number(pols, vrblvl)
    fail = int(deg != 9)
    deg = m_partition_bezout_number(pols, partition, vrblvl=vrblvl)
    fail = fail + int(deg != 9)
    start, sols = m_homogeneous_start_system(pols, partition, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
    fail = fail + int(len(sols) != 9)
    return fail

def test_linear_product_root_count(vrblvl=0):
    """
    Tests the linear product root count.
    """
    if vrblvl > 0:
        print('in test_linear_product_root_count ...')
    pols = noon3()
    lprc, sets = linear_product_root_count(pols, vrblvl=vrblvl)
    if vrblvl > 0:
        print('linear product root count of noon3 :', lprc)
        print('the supporting set structure :')
        print(sets)
    fail = int(lprc != 21)
    prodsys, prodsols = random_linear_product_system(pols, vrblvl=vrblvl)
    if vrblvl > 0:
        print('a random linear-product system :')
        for pol in prodsys:
            print(pol)
        print('the solutions :')
        for (idx, sol) in enumerate(prodsols):
            print('Solution', idx+1, ':')
            print(sol)
    fail = fail + int(len(prodsols) != 21)
    return fail

def main():
    """
    Runs some tests.
    """
    lvl = 1
    fail = test_total_degree(lvl)
    fail = fail + test_m_homogeneous_degree(lvl)
    fail = fail + test_linear_product_root_count(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()



from phc_edit import *
from phc_update import *

from time import sleep
from phc_config import solve_sleep_time

def hom_check(Uid, file_id_start, poly):
    """
    homotopy check, not finished
    """
    return 1


def PHC_Solve(Uid, Status, form, phcdb):
   if Status != None:
      PHC_Update(Uid,'0',"current",phcdb)
   else:
      if not form.has_key('poly'):
         PHC_Edit(Uid, form)
      else:
         Poly = form['poly'].value.strip()
         if form.has_key('homotopy'):
#            error_hom_check = hom_check(Uid, form['homotopy'].value, Poly)
#            
#            if not error_hom_check:
            data = PHC_Server(Uid,'7', 'current|' + form['homotopy'].value + '|'+ Poly)
#            else:
#               data = 0
#               PHC_Edit_Hom(Uid,form)
#               print "<p><font color='red'><b>Error:</b> These two polynomials doesn't match.</font></p>"
               
            if data != 0:
               print "<p>Start polynomial systems = %s</p>"%form['homotopy'].value
         else:
            data = PHC_Server(Uid,'1',Poly)

         if data != "error" and data != 0:
            phcdb.userstatus(Uid, "1")
            sleep(solve_sleep_time)
            PHC_Update(Uid,'0',"current",phcdb)



"""
The module trackers offers functions to track paths starting
at the known solutions of a start system and leading to
the desired solutions of a target system,
with aposteriori step size control algorithms.
An aposteriori step size control algorithm determines the step size
based on the performance of the corrector.
For small problems, the default values of the parameters and tolerances 
for predictor and corrector suffice, otherwise they must be tuned.
Reruns of paths must happen with the same value of the gamma constant.
"""
from ctypes import c_int32, c_double, pointer
from phcpy.version import get_phcfun
from phcpy.polynomials import number_of_symbols
from phcpy.solutions import set_double_solutions
from phcpy.solutions import set_double_double_solutions
from phcpy.solutions import set_quad_double_solutions
from phcpy.solutions import clear_double_solutions
from phcpy.solutions import clear_double_double_solutions
from phcpy.solutions import clear_quad_double_solutions
from phcpy.solutions import get_next_double_solution
from phcpy.solutions import get_next_double_double_solution
from phcpy.solutions import get_next_quad_double_solution
from phcpy.solutions import strsol2dict, verify
from phcpy.homotopies import set_double_target_system
from phcpy.homotopies import set_double_start_system
from phcpy.homotopies import set_double_laurent_target_system
from phcpy.homotopies import set_double_laurent_start_system
from phcpy.homotopies import set_double_start_solutions
from phcpy.homotopies import set_double_homotopy
from phcpy.homotopies import set_double_laurent_homotopy
from phcpy.homotopies import get_double_target_solutions
from phcpy.homotopies import clear_double_homotopy
from phcpy.homotopies import clear_double_laurent_homotopy
from phcpy.homotopies import set_double_double_target_system
from phcpy.homotopies import set_double_double_start_system
from phcpy.homotopies import set_double_double_laurent_target_system
from phcpy.homotopies import set_double_double_laurent_start_system
from phcpy.homotopies import set_double_double_start_solutions
from phcpy.homotopies import set_double_double_homotopy
from phcpy.homotopies import set_double_double_laurent_homotopy
from phcpy.homotopies import get_double_double_target_solutions
from phcpy.homotopies import clear_double_double_homotopy
from phcpy.homotopies import clear_double_double_laurent_homotopy
from phcpy.homotopies import set_quad_double_target_system
from phcpy.homotopies import set_quad_double_start_system
from phcpy.homotopies import set_quad_double_laurent_target_system
from phcpy.homotopies import set_quad_double_laurent_start_system
from phcpy.homotopies import set_quad_double_start_solutions
from phcpy.homotopies import set_quad_double_homotopy
from phcpy.homotopies import set_quad_double_laurent_homotopy
from phcpy.homotopies import get_quad_double_target_solutions
from phcpy.homotopies import clear_quad_double_homotopy
from phcpy.homotopies import clear_quad_double_laurent_homotopy
from phcpy.starters import total_degree_start_system

def show_parameters(vrblvl=0):
    """
    Displays the current values of the continuation parameters.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in show_parameters ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> show_parameters calls phc', end='')
    retval = phc(194, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def autotune_parameters(difficulty_level, digits_of_precision, vrblvl=0):
    """
    Tunes the parameters given the difficulty level of the homotopy
    and the digits of working precision.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in autotune_parameters, difficulty :', difficulty_level, end='')
        print(', digits :', digits_of_precision)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(difficulty_level))
    bbb = pointer(c_int32(digits_of_precision))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_parameter_value calls phc', end='')
    retval = phc(193, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def interactive_tune(vrblvl=0):
    """
    Interactive tuning of the parameters.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in interactive_tune ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_parameter_value calls phc', end='')
    retval = phc(70, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_parameter_value(idx, value, vrblvl=0):
    """
    Sets the parameter with index idx to the given value.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in set_parameter_value, idx :', idx, end='')
        print(', value :', value)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(value))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_parameter_value calls phc', end='')
    retval = phc(190, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def get_parameter_value(idx, vrblvl=0):
    """
    Returns the value of the parameter with index idx.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in get_parameter_value, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx))
    bbb = pointer(c_int32(0))
    value = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> set_parameter_value calls phc', end='')
    retval = phc(189, aaa, bbb, value, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
        print('Value of parameter', idx, ':', value[0])
    return value[0]

def write_parameters(vrblvl=0):
    """
    Writes the parameters with repeated calls to get_parameter_value(),
    as the show_parameters() does not work in a Jupyter notebook.
    """
    pars = [get_parameter_value(idx, vrblvl) for idx in range(35)]
    print('GLOBAL MONITOR :')
    print('  1. the condition of the homotopy           :', int(pars[1]))
    print('  2. number of paths tracked simultaneously  :', int(pars[2]))
    print('  3. maximum number of steps along a path    :', int(pars[3]))
    print('  4. distance from target to start end game  :', end='')
    print(f' {pars[4]:1.3e}')
    print('  5. order of extrapolator in end game       :', int(pars[5]))
    print('  6. maximum number of re-runs               :', int(pars[6]))
    print('STEP CONTROL (PREDICTOR) :                    along path : end game')
    s1s = '( x:Sec,t:Rea )' # secant for x, real for t
    s1c = '( x:Sec,t:Com )' # secant for x, complex for t
    s1g = '( x:Sec,t:Geo )' # second for x, geometric for t
    t1s = '( x:Tan,t:Rea )' # tangent for x, real for t
    t1c = '( x:Tan,t:Com )' # tangent for x, complex for t
    t1g = '( x:Tan,t:Geo )' # tangent for x, geometric for t
    h3s = '( x:Her,t:Rea )' # Hermite for x, real for t
    q2s = '( x:Qu2,t:Rea )' # quadratic for for x, real for t
    c3s = '( x:Cub,t:Rea )' # cubic for x, real for t
    ptp = [s1s, s1c, s1g, t1s, t1c, t1g, h3s, q2s, c3s]
    uuu = 'no predictor'
    if int(pars[7]) in range(9):
        if int(pars[8]) in range(9):
            print(f'  7: 8. type {ptp[int(pars[7])]}:{ptp[int(pars[8])]} :',
                  end=' ')
        else:
            print(f'  7: 8. type {ptp[int(pars[7])]}:{uuu} :', end=' ')
    else:
        if int(pars[8]) in range(9):
            print(f'  7: 8. type {uuu}:{ptp[int(pars[8])]} :',
                  end=' ')
        else:
            print(f'  7: 8. type {uuu}:{uuu} :', end=' ')
    print(int(pars[7]), '        :', int(pars[8]))
    print('  9:10. minimum step size                    :', end='')
    print(f' {pars[9]:1.3e} : {pars[10]:1.3e}')
    print(' 11:12. maximum step size                    :', end='')
    print(f' {pars[11]:1.3e} : {pars[12]:1.3e}')
    print(' 13:14. reduction factor for step size       :', end='')
    print(f' {pars[13]:1.3e} : {pars[14]:1.3e}')
    print(' 15:16. expansion factor for step size       :', end='')
    print(f' {pars[15]:1.3e} : {pars[16]:1.3e}')
    print(' 17:18. expansion threshold                  :', end=' ')
    print(int(pars[17]), '        :', int(pars[18]))
    print('PATH CLOSENESS (CORRECTOR) :                  along path : end game')
    print(' 19:20. maximum number of iterations         :', end=' ')
    print(int(pars[19]), '        :', int(pars[20]))
    print(' 21:22. relative precision for residuals     :', end='')
    print(f' {pars[21]:1.3e} : {pars[22]:1.3e}')
    print(' 23:24. absolute precision for residuals     :', end='')
    print(f' {pars[23]:1.3e} : {pars[24]:1.3e}')
    print(' 25:26. relative precision for corrections   :', end='')
    print(f' {pars[25]:1.3e} : {pars[26]:1.3e}')
    print(' 27:28. absolute precision for corrections   :', end='')
    print(f' {pars[27]:1.3e} : {pars[28]:1.3e}')
    print('SOLUTION TOLERANCES :                         along path : end game')
    print(' 29:30. inverse condition of Jacobian        :', end='')
    print(f' {pars[29]:1.3e} : {pars[30]:1.3e}')
    print(' 31:32. clustering of solutions              :', end='')
    print(f' {pars[31]:1.3e} : {pars[32]:1.3e}')
    print(' 33:34. solution at infinity                 :', end='')
    print(f' {pars[33]:1.3e} : {pars[34]:1.3e}')
    return 0

def set_condition_level(level, vrblvl=0):
    r"""
    Sets the parameter that represents the difficulty level of the
    homotopy to the value of *level*.  The default level equals zero,
    higher values lead to smaller tolerances.
    On return is the failure code, which is zero if all went well.
    """
    return set_parameter_value(1, level, vrblvl)

def get_condition_level(vrblvl=0):
    """
    Returns the level of difficulty.
    The verbose level is given by vrblvl.
    """
    return int(get_parameter_value(1, vrblvl))

def clear_double_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in double precision
    with an artificial parameter homotopy.
    """
    if vrblvl > 0:
        print('in clear_double_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_track_data calls phc', end='')
    retval = phc(18, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in double double precision
    with an artificial parameter homotopy.
    """
    if vrblvl > 0:
        print('in clear_double_double_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_track_data calls phc', end='')
    retval = phc(238, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in quad double precision
    with an artificial parameter homotopy.
    """
    if vrblvl > 0:
        print('in clear_quad_double_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_track_data calls phc', end='')
    retval = phc(248, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_laurent_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in double precision
    with an artificial parameter Laurent homotopy.
    """
    if vrblvl > 0:
        print('in clear_double_laurent_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_laurent_track_data calls phc', end='')
    retval = phc(771, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_laurent_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in double double precision
    with an artificial parameter Laurent homotopy.
    """
    if vrblvl > 0:
        print('in clear_double_double_laurent_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_laurent_track_data calls phc', end='')
    retval = phc(772, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_laurent_track_data(vrblvl=0):
    """
    Clears the data allocated for path tracking in quad double precision
    with an artificial parameter Laurent homotopy.
    """
    if vrblvl > 0:
        print('in clear_quad_double_laurent_track_data ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_laurent_track_data calls phc', end='')
    retval = phc(773, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_double_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in double precision with a number
    of tasks equal to tasks (no multithreading if zero).
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_double_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_double_track calls phc', end='')
    retval = phc(16, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_double_double_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in double double precision with a number
    of tasks equal to tasks (no multithreading if zero).
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_double_double_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_double_double_track calls phc', end='')
    retval = phc(236, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_quad_double_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in quad double precision with a number
    of tasks equal to tasks (no multithreading if zero).
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_quad_double_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_quad_double_track calls phc', end='')
    retval = phc(246, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_double_laurent_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in double precision with a number
    of tasks equal to tasks (no multithreading if zero),
    for a homotopy of Laurent polynomial systems.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_double_laurent_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_double_laurent_track calls phc', end='')
    retval = phc(774, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_double_double_laurent_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in double double precision with a number
    of tasks equal to tasks (no multithreading if zero),
    for a homotopy of Laurent polynomial sysetms.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_double_double_laurent_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_double_double_laurent_track calls phc', end='')
    retval = phc(775, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def do_quad_double_laurent_track(tasks=0, vrblvl=0):
    """
    Calls the path trackers in quad double precision with a number
    of tasks equal to tasks (no multithreading if zero),
    for a homotopy of Laurent polynomial systems.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in do_quad_double_laurent_track, tasks :', tasks)
    phc = get_phcfun(vrblvl-1)
    nbtasks = pointer(c_int32(tasks))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> do_quad_double_laurent_track calls phc', end='')
    retval = phc(776, nbtasks, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def double_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Track paths in double precision.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    """
    if vrblvl > 0:
        print('in double_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_double_target_system(target, vrblvl-1)
    set_double_start_system(start, vrblvl-1)
    nvr = number_of_symbols(start, vrblvl-1)
    set_double_start_solutions(nvr, startsols, vrblvl-1)
    usedgamma = set_double_homotopy(gamma, pwt, vrblvl-1)
    do_double_track(tasks, vrblvl)
    sols = get_double_target_solutions(vrblvl-1)
    clear_double_solutions(vrblvl-1)
    clear_double_homotopy(vrblvl-1)
    clear_double_track_data(vrblvl)
    return (usedgamma, sols)

def double_double_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Tracks paths in double double precision.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    Note: tasks=0 does not work ...
    """
    if vrblvl > 0:
        print('in double_double_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_double_double_target_system(target, vrblvl-1)
    set_double_double_start_system(start, vrblvl-1)
    nvr = number_of_symbols(start, vrblvl-1)
    set_double_double_start_solutions(nvr, startsols, vrblvl-1)
    usedgamma = set_double_double_homotopy(gamma, pwt, vrblvl-1)
    do_double_double_track(tasks, vrblvl)
    sols = get_double_double_target_solutions(vrblvl-1)
    clear_double_double_solutions(vrblvl-1)
    clear_double_double_homotopy(vrblvl-1)
    clear_double_double_track_data(vrblvl)
    return (usedgamma, sols)

def quad_double_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Tracks paths in quad double precision.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    Note: tasks=0 does not work ...
    """
    if vrblvl > 0:
        print('in quad_double_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_quad_double_target_system(target, vrblvl-1)
    set_quad_double_start_system(start, vrblvl-1)
    nvr = number_of_symbols(start, vrblvl-1)
    set_quad_double_start_solutions(nvr, startsols, vrblvl-1)
    usedgamma = set_quad_double_homotopy(gamma, pwt, vrblvl-1)
    do_quad_double_track(tasks, vrblvl)
    sols = get_quad_double_target_solutions(vrblvl-1)
    clear_quad_double_homotopy(vrblvl-1)
    clear_quad_double_track_data(vrblvl)
    return (usedgamma, sols)

def double_laurent_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Track paths in double precision, for Laurent systems.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    """
    if vrblvl > 0:
        print('in double_laurent_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_double_laurent_target_system(target, vrblvl)
    set_double_laurent_start_system(start, vrblvl)
    nvr = number_of_symbols(start)
    set_double_start_solutions(nvr, startsols, vrblvl)
    usedgamma = set_double_laurent_homotopy(gamma, pwt, vrblvl)
    do_double_laurent_track(tasks, vrblvl)
    sols = get_double_target_solutions(vrblvl)
    clear_double_solutions(vrblvl)
    clear_double_laurent_homotopy(vrblvl)
    clear_double_laurent_track_data(vrblvl)
    return (usedgamma, sols)

def double_double_laurent_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Tracks paths in double double precision, for Laurent systems.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    Note: tasks=0 does not work ...
    """
    if vrblvl > 0:
        print('in double_double_laurent_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_double_double_laurent_target_system(target, vrblvl-1)
    set_double_double_laurent_start_system(start, vrblvl-1)
    nvr = number_of_symbols(start, vrblvl-1)
    set_double_double_start_solutions(nvr, startsols, vrblvl-1)
    usedgamma = set_double_double_laurent_homotopy(gamma, pwt, vrblvl-1)
    do_double_double_laurent_track(tasks, vrblvl)
    sols = get_double_double_target_solutions(vrblvl-1)
    clear_double_double_solutions(vrblvl-1)
    clear_double_double_laurent_homotopy(vrblvl-1)
    clear_double_double_laurent_track_data(vrblvl)
    return (usedgamma, sols)

def quad_double_laurent_track(target, start, startsols, \
    gamma=0, pwt=2, tasks=0, vrblvl=0):
    r"""
    Tracks paths in quad double precision, for Laurent systems.
    On input are a target system, a start system with solutions,
    optionally: a (random) gamma constant and the number of tasks.
    The *target* is a list of strings representing the polynomials
    of the target system (which has to be solved).
    The *start* is a list of strings representing the polynomials
    of the start system, with known solutions in sols.
    The *startsols* is a list of strings representing start solutions.
    By default, a random *gamma* constant is generated,
    otherwise *gamma* must be a nonzero complex constant.
    The *pwt* is the power of t in the homotopy.
    Changing the default of *pwt* can only be done if a nonzero complex
    value for *gamma* is provided as well.
    The number of tasks in the multithreading is defined by *tasks*.
    The default zero value for *tasks* indicates no multithreading.
    On return is a tuple, with first the gamma used in the homotopy
    and then second, the string representations of the solutions
    computed at the end of the paths.
    Note: tasks=0 does not work ...
    """
    if vrblvl > 0:
        print('in quad_double_laurent_track, with gamma :', gamma, end='')
        print(', pwt :', pwt, end='')
        print(', tasks :', tasks)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    set_quad_double_laurent_target_system(target, vrblvl-1)
    set_quad_double_laurent_start_system(start, vrblvl-1)
    nvr = number_of_symbols(start, vrblvl-1)
    set_quad_double_start_solutions(nvr, startsols, vrblvl-1)
    usedgamma = set_quad_double_laurent_homotopy(gamma, pwt, vrblvl-1)
    do_quad_double_laurent_track(tasks, vrblvl)
    sols = get_quad_double_target_solutions(vrblvl-1)
    clear_quad_double_laurent_homotopy(vrblvl-1)
    clear_quad_double_laurent_track_data(vrblvl)
    return (usedgamma, sols)

def initialize_double_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0, vrblvl=0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in standard double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    if vrblvl > 0:
        print('in initialize_double_tracker', end='')
        print(', fixedgamma :', fixedgamma, end='')
        print(', regamma :', regamma, end='')
        print(', imgamma :', imgamma)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
    set_double_target_system(target, vrblvl)
    set_double_start_system(start, vrblvl)
    phc = get_phcfun(vrblvl-1)
    afix = pointer(c_int32(int(fixedgamma)))
    bbb = pointer(c_int32(0))
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(regamma)
    c_gamma[1] = c_double(imgamma)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_tracker calls phc', end='')
    retval = phc(500, afix, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_double_double_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0, vrblvl=0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in double double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    if vrblvl > 0:
        print('in initialize_double_double_tracker', end='')
        print(', fixedgamma :', fixedgamma, end='')
        print(', regamma :', regamma, end='')
        print(', imgamma :', imgamma)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
    set_double_double_target_system(target, vrblvl)
    set_double_double_start_system(start, vrblvl)
    phc = get_phcfun(vrblvl-1)
    afix = pointer(c_int32(int(fixedgamma)))
    bbb = pointer(c_int32(0))
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(regamma)
    c_gamma[1] = c_double(imgamma)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_double_tracker calls phc', end='')
    retval = phc(501, afix, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_quad_double_tracker(target, start, fixedgamma=True, \
    regamma=0.0, imgamma=0.0, vrblvl=0):
    r"""
    Initializes a path tracker with a generator for a *target*
    and *start* system given in quad double precision.
    If *fixedgamma*, then gamma will be a fixed default value,
    otherwise, a random complex constant for gamma is generated,
    but only if *regamma* and *imgamma* are both equal to 0.0.
    If not *fixedgamma* and moreover: *regamma* and *imgamma* are not
    both zero, then the complex number with real part in *regamma*
    and imaginary part in *imgamma* will be the gamma constant.
    """
    if vrblvl > 0:
        print('in initialize_quad_double_tracker', end='')
        print(', fixedgamma :', fixedgamma, end='')
        print(', regamma :', regamma, end='')
        print(', imgamma :', imgamma)
        print('the target system :')
        for pol in target:
            print(pol)
        print('the start system :')
        for pol in start:
            print(pol)
    set_quad_double_target_system(target, vrblvl)
    set_quad_double_start_system(start, vrblvl)
    phc = get_phcfun(vrblvl-1)
    afix = pointer(c_int32(int(fixedgamma)))
    bbb = pointer(c_int32(0))
    c_gamma = (c_double*2)()
    c_gamma[0] = c_double(regamma)
    c_gamma[1] = c_double(imgamma)
    ptr_gamma = pointer(c_gamma)
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_quad_double_tracker calls phc', end='')
    retval = phc(502, afix, bbb, ptr_gamma, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_double_solution(nvr, sol, vrblvl=0):
    r"""
    A double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvr*.
    """
    if vrblvl > 0:
        print('in initialize_double_solution, nvr :', nvr)
        print('the solution :')
        print(sol)
    set_double_solutions(nvr, [sol], vrblvl)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_solution calls phc', end='')
    retval = phc(503, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_double_double_solution(nvr, sol, vrblvl=0):
    r"""
    A double double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvr*.
    """
    if vrblvl > 0:
        print('in initialize_double_double_solution, nvr :', nvr)
        print('the solution :')
        print(sol)
    set_double_double_solutions(nvr, [sol], vrblvl)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_double_double_solution calls phc', end='')
    retval = phc(504, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def initialize_quad_double_solution(nvr, sol, vrblvl=0):
    r"""
    A quad double precision path tracker with a generator is
    initialized with a start solution *sol* in a number of
    variables equal to the value of *nvr*.
    """
    if vrblvl > 0:
        print('in initialize_quad_double_solution, nvr :', nvr)
        print('the solution :')
        print(sol)
    set_quad_double_solutions(nvr, [sol], vrblvl)
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> initialize_quad_double_solution calls phc', end='')
    retval = phc(505, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def next_double_solution(vrblvl=0):
    r"""
    Returns the next solution on a path tracked with
    double precision arithmetic, provided the functions
    **initialize_double_tracker()** and
    **initialize_double_solution()** have been executed properly.
    """
    if vrblvl > 0:
        print('in next_double_solution ...')
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> next_double_solution calls phc', end='')
    retval = phc(506, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    sol = get_next_double_solution(1, vrblvl)
    return sol

def next_double_double_solution(vrblvl=0):
    r"""
    Returns the next solution on a path tracked with
    double double precision arithmetic, provided the functions
    **initialize_double_double_tracker()** and
    **initialize_double_double_solution()** have been executed properly.
    """
    if vrblvl > 0:
        print('in next_double_double_solution ...')
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> next_double_double_solution calls phc', end='')
    retval = phc(507, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    sol = get_next_double_double_solution(1, vrblvl)
    return sol

def next_quad_double_solution(vrblvl=0):
    r"""
    Returns the next solution on a path tracked with
    quad double precision arithmetic, provided the functions
    **initialize_quad_double_tracker()** and
    **initialize_quad_double_solution()** have been executed properly.
    """
    if vrblvl > 0:
        print('in next_quad_double_solution ...')
    phc = get_phcfun(vrblvl-1)
    aidx = pointer(c_int32(1))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> next_quad_double_solution calls phc', end='')
    retval = phc(508, aidx, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    sol = get_next_quad_double_solution(1, vrblvl)
    return sol

def test_double_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    in double precision.
    """
    if vrblvl > 0:
        print('in test_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = double_track(mickey, start, startsols, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_double_double_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = double_double_track(mickey, start, startsols, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_quad_double_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = quad_double_track(mickey, start, startsols, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_next_double_track(vrblvl=0):
    """
    Tests the step-by-step tracking on the mickey mouse example
    of two quadrics, in double precision.
    """
    if vrblvl > 0:
        print('in test_next_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    initialize_double_tracker(mickey, start, vrblvl-1)
    initialize_double_solution(2, startsols[0], vrblvl-1)
    autotune_parameters(0, 15, vrblvl-1)
    fail = 1
    for step in range(1, 101):
        sol = next_double_solution(vrblvl-1)
        if vrblvl > 0:
            print('the next solution :')
            print(sol)
        dsol = strsol2dict(sol)
        if vrblvl > 0:
            print('step', step, end='')
            print(', t :', dsol['t'].real)
        if dsol['t'].real >= 1.0:
            if vrblvl > 0:
                print('the end solution :')
                print(sol)
            fail = 0
            break
    clear_double_solutions(vrblvl-1)
    return fail

def test_next_double_double_track(vrblvl=0):
    """
    Tests the step-by-step tracking on the mickey mouse example
    of two quadrics, in double double precision.
    """
    if vrblvl > 0:
        print('in test_next_double_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    initialize_double_double_tracker(mickey, start, vrblvl-1)
    initialize_double_double_solution(2, startsols[0], vrblvl-1)
    autotune_parameters(0, 15, vrblvl-1)
    fail = 1
    for step in range(1, 101):
        sol = next_double_double_solution(vrblvl-1)
        if vrblvl > 0:
            print('the next solution :')
            print(sol)
        dsol = strsol2dict(sol)
        if vrblvl > 0:
            print('step', step, end='')
            print(', t :', dsol['t'].real)
        if dsol['t'].real >= 1.0:
            if vrblvl > 0:
                print('the end solution :')
                print(sol)
            fail = 0
            break
    clear_double_solutions(vrblvl-1)
    clear_double_double_solutions(vrblvl-1)
    return fail

def test_next_quad_double_track(vrblvl=0):
    """
    Tests the step-by-step tracking on the mickey mouse example
    of two quadrics, in quad double precision.
    """
    if vrblvl > 0:
        print('in test_next_quad_double_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    initialize_quad_double_tracker(mickey, start, vrblvl-1)
    initialize_quad_double_solution(2, startsols[0], vrblvl-1)
    autotune_parameters(0, 15, vrblvl-1)
    fail = 1
    for step in range(1, 101):
        sol = next_quad_double_solution(vrblvl-1)
        if vrblvl > 0:
            print('the next solution :')
            print(sol)
        dsol = strsol2dict(sol)
        if vrblvl > 0:
            print('step', step, end='')
            print(', t :', dsol['t'].real)
        if dsol['t'].real >= 1.0:
            if vrblvl > 0:
                print('the end solution :')
                print(sol)
            fail = 0
            break
    clear_double_solutions(vrblvl-1)
    clear_quad_double_solutions(vrblvl-1)
    return fail

def test_double_laurent_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    set as Laurent system in double precision.
    """
    if vrblvl > 0:
        print('in test_double_laurent_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = double_laurent_track(mickey, start, startsols, \
        vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_double_double_laurent_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    set as Laurent system in double double precision.
    """
    if vrblvl > 0:
        print('in test_double_double_laurent_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = double_double_laurent_track(mickey, start, startsols, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_quad_double_laurent_track(vrblvl=0):
    """
    Tests tracking the mickey mouse example of two quadrics,
    set as Laurent system in quad double precision.
    """
    if vrblvl > 0:
        print('in test_quad_double_laurent_track ...')
    mickey = ['x^2 + 4*y^2 - 4;', '2*y^2 - x;']
    start, startsols = total_degree_start_system(mickey, vrblvl=vrblvl)
    if vrblvl > 0:
        print('the start system :')
        for pol in start:
            print(pol)
        print('the start solutions :')
        for (idx, sol) in enumerate(startsols):
            print('Solution', idx+1, ':')
            print(sol)
    gamma, sols = quad_double_laurent_track(mickey, start, startsols, \
        vrblvl=vrblvl)
    if vrblvl > 0:
        print('gamma :', gamma)
        print('the solutions :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    err = verify(mickey, sols, vrblvl-1)
    if vrblvl > 0:
        print('the error sum :', err)
    if len(sols) == 4 and err < 1.0e-10:
        if vrblvl > 0:
            print('Found 4 solutions and error is okay.')
        return 0
    if len(sols) != 4:
        if vrblvl > 0:
            print('Number of solutions is not 4 :', len(sols))
        return 1
    if err >= 1.0e-10:
        if vrblvl > 0:
            print('The error is too large.')
    return 1

def test_tuning(vrblvl=0):
    """
    Runs some tests on tuning the parameters.
    """
    if vrblvl > 0:
        print('in test_tuning ...')
        show_parameters(vrblvl-1)
        print('setting the condition level to 2 ...')
    fail = set_condition_level(2, vrblvl-1)
    level = get_condition_level(vrblvl-1)
    if vrblvl > 0:
        print('the condition level :', level)
    fail = fail + autotune_parameters(level, 14, vrblvl-1)
    if vrblvl > 0:
        show_parameters(vrblvl-1)
    # interactive_tune(vrblvl)
    fail = fail + autotune_parameters(0, 14, vrblvl-1)
    if vrblvl > 0:
        show_parameters(vrblvl)
    return fail

def test_write_parameters(vrblvl=0):
    """
    Tests the writing of the parameters.
    """
    if vrblvl > 0:
        show_parameters(vrblvl)
        write_parameters(vrblvl)
    return 0

def main():
    """
    Runs some tests on tuning and tracking.
    """
    lvl = 1
    fail = test_tuning(lvl)
    fail = fail + test_double_track(lvl)
    fail = fail + test_double_double_track(lvl)
    fail = fail + test_quad_double_track(lvl)
    fail = fail + test_next_double_track(lvl)
    fail = fail + test_next_double_double_track(lvl)
    fail = fail + test_next_quad_double_track(lvl)
    fail = fail + test_double_laurent_track(lvl)
    fail = fail + test_double_double_laurent_track(lvl)
    fail = fail + test_quad_double_laurent_track(lvl)
    fail = fail + test_write_parameters(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()
